@@ -6,12 +6,9 @@
     :synopsis: module that contains config recognizer classes.
 """
 
-import re
-
 from mockintosh.constants import PYBARS, JINJA
 from mockintosh.templating import TemplateRenderer
 from mockintosh.params import (
-    PathParam,
     HeaderParam,
     QueryStringParam,
     BodyTextParam,
@@ -54,38 +51,45 @@ class RecognizerBase():
                 new_parts = {}
 
             for key, value in parts.items():
-                var, new_part, context = self.render_part(key, value)
-                if var is not None:
-                    param = None
+                if not isinstance(value, list):
+                    value = [value]
+                for el in value:
+                    context = {}
+                    if self.scope != 'path':
+                        _var, key, context = self.render_part(key, key)
+                    var, new_part, _context = self.render_part(key, el)
+                    context.update(_context)
+                    if var is not None:
+                        param = None
+                        if self.scope == 'headers':
+                            param = HeaderParam(key, var)
+                        elif self.scope == 'queryString':
+                            param = QueryStringParam(key, var)
+                        elif self.scope == 'bodyUrlencoded':
+                            param = BodyUrlencodedParam(key, var)
+                        elif self.scope == 'bodyMultipart':
+                            param = BodyMultipartParam(key, var)
+                        self.params[var] = param
+                        if self.scope == 'path':
+                            priority = 2
+                    else:
+                        if self.scope == 'queryString':
+                            param = QueryStringParam(_var, key)
+                            self.params[key] = param
                     if self.scope == 'path':
-                        param = PathParam(key, var)
-                    elif self.scope == 'headers':
-                        param = HeaderParam(key, var)
-                    elif self.scope == 'queryString':
-                        param = QueryStringParam(key, var)
-                    elif self.scope == 'bodyUrlencoded':
-                        param = BodyUrlencodedParam(key, var)
-                    elif self.scope == 'bodyMultipart':
-                        param = BodyMultipartParam(key, var)
-                    self.params[var] = param
-                    if self.scope == 'path':
-                        priority = 2
-                if self.scope == 'path':
-                    if priority == 0 and new_part != value:
-                        priority = 1
-                    new_parts.append(new_part)
-                else:
-                    new_parts[key] = new_part
-                self.all_contexts.update(context)
+                        if priority == 0 and new_part != el:
+                            priority = 1
+                        new_parts.append(new_part)
+                    else:
+                        new_parts[key] = new_part
+                    self.all_contexts.update(context)
 
             if self.scope == 'path':
                 return '/'.join(new_parts), priority
             else:
                 return new_parts
 
-    def render_part(self, key, text):
-        var = None
-
+    def render_part(self, key: str, text: str):
         if self.engine == PYBARS:
             from mockintosh.hbs.methods import reg_ex
         elif self.engine == JINJA:
@@ -101,26 +105,18 @@ class RecognizerBase():
             self.engine,
             text,
             inject_objects=inject_objects,
-            inject_methods=[reg_ex]
+            inject_methods=[reg_ex],
+            fill_undefineds_with='([^/]+)' if self.scope == 'path' else '(.*)'
         )
         compiled, context = renderer.render()
         if self.engine == PYBARS:
             del context['scope']
             if 'key' in inject_objects:
                 del context['key']
+        for key in renderer.keys_to_delete:
+            context[key] = None
 
-        if not compiled:
-            match = re.search(r'{{(.*)}}', text)
-            if match is not None:
-                name = match.group(1).strip()
-                if self.scope == 'path':
-                    compiled = '[^/]+'
-                else:
-                    compiled = '.*'
-                var = name
-            else:
-                compiled = text
-        return var, compiled, context
+        return renderer.one_and_only_var, compiled, context
 
 
 class PathRecognizer(RecognizerBase):
