@@ -8,9 +8,11 @@
 
 import logging
 import sys
+import traceback
 from abc import abstractmethod
-from os import path, environ
 from collections import OrderedDict
+from os import path, environ
+from typing import Union
 
 import tornado.ioloop
 import tornado.web
@@ -18,18 +20,22 @@ from tornado.routing import Rule, RuleRouter, HostMatches
 
 from mockintosh.exceptions import CertificateLoadingError
 from mockintosh.handlers import GenericHandler
+from mockintosh.logs import Logs
 from mockintosh.management import (
     ManagementRootHandler,
     ManagementConfigHandler,
     ManagementStatsHandler,
+    ManagementLogsHandler,
     ManagementResetIteratorsHandler,
     ManagementUnhandledHandler,
     ManagementOasHandler,
     ManagementTagHandler,
+    ManagementResourcesHandler,
     ManagementServiceRootHandler,
     ManagementServiceRootRedirectHandler,
     ManagementServiceConfigHandler,
     ManagementServiceStatsHandler,
+    ManagementServiceLogsHandler,
     ManagementServiceResetIteratorsHandler,
     ManagementServiceUnhandledHandler,
     ManagementServiceOasHandler,
@@ -39,6 +45,7 @@ from mockintosh.management import (
 from mockintosh.stats import Stats
 
 stats = Stats()
+logs = Logs()
 
 __location__ = path.abspath(path.dirname(__file__))
 
@@ -46,11 +53,11 @@ __location__ = path.abspath(path.dirname(__file__))
 class Impl:
     @abstractmethod
     def get_server(self, app, is_ssl, ssl_options):
-        pass
+        raise NotImplementedError
 
     @abstractmethod
     def serve(self):
-        pass
+        raise NotImplementedError
 
 
 class TornadoImpl(Impl):
@@ -63,12 +70,23 @@ class TornadoImpl(Impl):
         return server
 
     def serve(self):
-        tornado.ioloop.IOLoop.current().start()
+        try:
+            tornado.ioloop.IOLoop.current().start()
+        except KeyboardInterrupt:
+            logging.debug("Shutdown: %s", traceback.format_exc())
+
+
+class _Listener:
+    def __init__(self, hostname: Union[str, None], port: int, address: Union[str, None]):
+        self.hostname = hostname
+        self.port = port
+        self.address = address
 
 
 class _Apps:
     def __init__(self):
         self.apps = []
+        self.listeners = []
 
 
 class HttpServer:
@@ -84,6 +102,7 @@ class HttpServer:
         self.services_log = []
         self._apps = _Apps()
         self.stats = stats
+        self.logs = logs
         self.unhandled_data = UnhandledData()
         self.load()
 
@@ -97,15 +116,15 @@ class HttpServer:
             service_id_counter += 1
 
             self.unhandled_data.requests.append({})
-            self.stats.add_service(
-                '%s:%s%s' % (
-                    service['hostname'] if 'hostname' in service else (
-                        self.address if self.address else 'localhost'
-                    ),
-                    service['port'],
-                    ' - %s' % service['name'] if 'name' in service else ''
-                )
+            hint = '%s:%s%s' % (
+                service['hostname'] if 'hostname' in service else (
+                    self.address if self.address else 'localhost'
+                ),
+                service['port'],
+                ' - %s' % service['name'] if 'name' in service else ''
             )
+            self.stats.add_service(hint)
+            self.logs.add_service(service['name'] if 'name' in service else '')
 
         port_mapping = OrderedDict()
         for service in self.definition.data['services']:
@@ -142,12 +161,12 @@ class HttpServer:
                     if 'name' in service:
                         if service['name'] not in self.services_list:
                             continue
-                    else:
-                        continue
+                    else:  # pragma: no cover
+                        continue  # https://github.com/nedbat/coveragepy/issues/198
 
                 endpoints = []
                 if 'endpoints' in service:
-                    endpoints = HttpServer.merge_alternatives(service, self.stats)
+                    endpoints = HttpServer.merge_alternatives(service, self.stats, self.logs)
 
                 management_root = None
                 if 'managementRoot' in service:
@@ -155,6 +174,13 @@ class HttpServer:
 
                 app = self.make_app(service, endpoints, self.globals, debug=self.debug, management_root=management_root)
                 self._apps.apps.append(app)
+                self._apps.listeners.append(
+                    _Listener(
+                        service['hostname'] if 'hostname' in service else None,
+                        service['port'],
+                        self.address if self.address else 'localhost'
+                    )
+                )
 
                 if 'hostname' not in service:
                     server = self.impl.get_server(app, ssl, ssl_options)
@@ -195,19 +221,18 @@ class HttpServer:
         self.load_management_api()
 
     @staticmethod
-    def merge_alternatives(service, stats):
+    def merge_alternatives(service: dict, stats: Stats, logs: Logs):
         new_endpoints = {}
         i = 0
         for endpoint in service['endpoints']:
             if 'method' not in endpoint:
                 endpoint['method'] = 'GET'
-            stats.services[service['internalServiceId']].add_endpoint(
-                '%s %s%s' % (
-                    endpoint['method'].upper(),
-                    endpoint['internalOrigPath'],
-                    ' - %s' % endpoint['id'] if 'id' in endpoint else ''
-                )
+            hint = '%s %s%s' % (
+                endpoint['method'].upper(),
+                endpoint['internalOrigPath'],
+                ' - %s' % endpoint['id'] if 'id' in endpoint else ''
             )
+            stats.services[service['internalServiceId']].add_endpoint(hint)
             identifier = endpoint['path']
             extracted_parts = {}
             for key in endpoint:
@@ -262,6 +287,7 @@ class HttpServer:
                 r'.*',
                 GenericHandler,
                 dict(
+                    http_server=self,
                     config_dir=self.definition.source_dir,
                     service_id=service['internalServiceId'],
                     endpoints=merged_endpoints,
@@ -269,7 +295,9 @@ class HttpServer:
                     definition_engine=self.definition.template_engine,
                     interceptors=self.interceptors,
                     stats=self.stats,
+                    logs=self.logs,
                     unhandled_data=self.unhandled_data if unhandled_enabled else None,
+                    fallback_to=service['fallbackTo'] if 'fallbackTo' in service else None,
                     tag=None
                 )
             )
@@ -309,6 +337,14 @@ class HttpServer:
                         service_id=service['internalServiceId']
                     )
                 ),
+                (
+                    '/%s/traffic-log' % management_root,
+                    ManagementServiceLogsHandler,
+                    dict(
+                        logs=logs,
+                        service_id=service['internalServiceId']
+                    )
+                ),
                 (
                     '/%s/reset-iterators' % management_root,
                     ManagementServiceResetIteratorsHandler,
@@ -340,6 +376,13 @@ class HttpServer:
                         http_server=self,
                         service_id=service['internalServiceId']
                     )
+                ),
+                (
+                    '/%s/resources' % management_root,
+                    ManagementResourcesHandler,
+                    dict(
+                        http_server=self
+                    )
                 )
             ] + endpoint_handlers
 
@@ -354,9 +397,9 @@ class HttpServer:
 
     def resolve_cert_path(self, cert_path):
         relative_path = path.join(self.definition.source_dir, cert_path)
+        relative_path = path.abspath(relative_path)
         if not path.isfile(relative_path):
             raise CertificateLoadingError('File not found on path `%s`' % cert_path)
-        relative_path = path.abspath(relative_path)
         if not relative_path.startswith(self.definition.source_dir):
             raise CertificateLoadingError('Path `%s` is inaccessible!' % cert_path)
 
@@ -404,6 +447,13 @@ class HttpServer:
                         stats=stats
                     )
                 ),
+                (
+                    '/traffic-log',
+                    ManagementLogsHandler,
+                    dict(
+                        logs=logs
+                    )
+                ),
                 (
                     '/reset-iterators',
                     ManagementResetIteratorsHandler,
@@ -431,6 +481,13 @@ class HttpServer:
                     dict(
                         http_server=self
                     )
+                ),
+                (
+                    '/resources',
+                    ManagementResourcesHandler,
+                    dict(
+                        http_server=self
+                    )
                 )
             ])
             server = self.impl.get_server(app, ssl, ssl_options)
